#include <qspinbox.h>

#include "adsrwidget.h"

#include "error.h"
#include "envelope.h"

/* 
 *  Constructs a ADSRWidget which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
ADSRWidget::ADSRWidget( QWidget* parent,  const char* name, WFlags fl )
  : UI_ADSRWidget( parent, name, fl ), Observer(NULL)
{

  adsr = NULL;
  sample = NULL;

  knobALev->setRange(0, 127);
  knobSLev->setRange(0, 127);

  knobATime->setRange(0, 1000);
  knobDTime->setRange(0, 1000);
  knobRTime->setRange(0, 1000);

  connect(sbAttackTime, SIGNAL(valueChanged(int)),
	  this, SLOT(attackTimeChanged(int)));
  connect(knobATime, SIGNAL(valueChanged(int)),
	  this, SLOT(attackTimeChanged(int)));
  connect(sbDecayTime, SIGNAL(valueChanged(int)),
	  this, SLOT(decayTimeChanged(int)));
  connect(knobDTime, SIGNAL(valueChanged(int)),
	  this, SLOT(decayTimeChanged(int)));
  connect(sbReleaseTime, SIGNAL(valueChanged(int)),
	  this, SLOT(releaseTimeChanged(int)));
  connect(knobRTime, SIGNAL(valueChanged(int)),
	  this, SLOT(releaseTimeChanged(int)));

  connect(sbAttackLevel, SIGNAL(valueChanged(int)),
	  this, SLOT(attackLevelChanged(int)));
  connect(knobALev, SIGNAL(valueChanged(int)),
	  this, SLOT(attackLevelChanged(int)));
  connect(sbSustainLevel, SIGNAL(valueChanged(int)),
	  this, SLOT(sustainLevelChanged(int)));
  connect(knobSLev, SIGNAL(valueChanged(int)),
	  this, SLOT(sustainLevelChanged(int)));

}

/*  
 *  Destroys the object and frees any allocated resources
 */
ADSRWidget::~ADSRWidget()
{
    // no need to delete child widgets, Qt does it all for us

  if (adsr) adsr->removeObserver(this);
}



void ADSRWidget::subjectDeleted(Subject *s) {

  if (s == adsr) adsr = NULL;
  update(NULL);
}



void ADSRWidget::setSample(Sample *s) {

  if(adsr) adsr->removeObserver(this);

  sample = s;
  if (!sample) adsr = NULL;
  else adsr = &sample->adsr;

  if(adsr) adsr->addObserver(this);  

  update(adsr);
}


void ADSRWidget::update(Subject *s) {

  if (s && (s == adsr)) {

    cAttackTime = adsr->getAttackTime();
    cDecayTime = adsr->getDecayTime();
    cReleaseTime = adsr->getReleaseTime();

    cAttackLevel = adsr->getAttackLevel();
    cSustainLevel = adsr->getSustainLevel();

    if (sbAttackTime->value() != cAttackTime) 
      sbAttackTime->setValue(cAttackTime);
    if (knobATime->getValue() != cAttackTime) 
      knobATime->setValue(cAttackTime);
    
    if (sbDecayTime->value() != cDecayTime) 
      sbDecayTime->setValue(cDecayTime);
    if (knobDTime->getValue() != cDecayTime) 
      knobDTime->setValue(cDecayTime);
    
    if (sbReleaseTime->value() != cReleaseTime) 
      sbReleaseTime->setValue(cReleaseTime);
    if (knobRTime->getValue() != cReleaseTime) 
      knobRTime->setValue(cReleaseTime);

    if (sbAttackLevel->value() != cAttackLevel) 
      sbAttackLevel->setValue(cAttackLevel);
    if (knobALev->getValue() != cAttackLevel) 
      knobALev->setValue(cAttackLevel);

    if (sbSustainLevel->value() != cSustainLevel) 
      sbSustainLevel->setValue(cSustainLevel);
    if (knobSLev->getValue() != cSustainLevel) 
      knobSLev->setValue(cSustainLevel);

  }
}


void ADSRWidget::attackTimeChanged(int v) {

  if (!sample) return;
  if (v != cAttackTime) adsr->setAttackTime(v);
}

void ADSRWidget::decayTimeChanged(int v) {

  if (!sample) return;
  if (v != cDecayTime) adsr->setDecayTime(v);
}

void ADSRWidget::releaseTimeChanged(int v) {

  if (!sample) return;
  if (v != cReleaseTime) adsr->setReleaseTime(v);
}

void ADSRWidget::attackLevelChanged(int v) {

  if (!sample) return;
  if (v != cAttackLevel) adsr->setAttackLevel(v);
}

void ADSRWidget::sustainLevelChanged(int v) {

  if (!sample) return;
  if (v != cSustainLevel) adsr->setSustainLevel(v);
}

