#include "instrumentselector.h"
#include "error.h"

/*
const char* chanLabels[17] = {"ch01", "ch02", "ch03", "ch04", "ch05", "ch06", 
			      "ch07", "ch08", "ch09", "ch10", "ch11", "ch12", 
			      "ch13", "ch14", "ch15", "ch16", "ch??" };
*/

/* 
 *  Constructs a InstrumentSelector which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
InstrumentSelector::InstrumentSelector(QWidget* parent, const char* name, WFlags fl) 
  : InstrumentSelectorUI(parent, name, fl), Observer(NULL) {

  instrument = Instrument::getInstrument(0);
  instrument->addObserver(this);

  // we need to keep informed about the output list
  Output::outputSubject.addObserver(this);

  buildOutputList();

   
  connect(cbChan, SIGNAL(activated(int)), 
	  this, SLOT(channelSelected(int)));

  connect(sbNum, SIGNAL(valueChanged(int)), 
	  this, SLOT(instrumentSelected(int)));

  connect(leName, SIGNAL(returnPressed()), 
	  this, SLOT(nameEntered()));

  connect(cbOutput, SIGNAL(activated(int)),
	  this, SLOT(outputSelected(int)));


  update(instrument);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
InstrumentSelector::~InstrumentSelector() {
  
  // no need to delete child widgets, Qt does it all for us
}


void InstrumentSelector::subjectDeleted(Subject *s) {

  // instruments don't get deleted, nothing to do
}


void InstrumentSelector::instrumentSelected(int i) {

  instrument->removeObserver(this);
  instrument = Instrument::getInstrument(i);
  instrument->addObserver(this);

  update(instrument);

  emit(instrumentChanged(instrument));
}


void InstrumentSelector::channelSelected(int i) {

  if (i == 16) i = -1;
  instrument->setMidiChannel(i);
}


void InstrumentSelector::nameEntered() {

  instrument->setName(leName->text());
}

void InstrumentSelector::outputSelected(int oNum) {
  
  instrument->setOutputNum(oNum);
}


void InstrumentSelector::buildOutputList() {

  int selected = cbOutput->currentItem();
  
  cbOutput->clear();
  
  int n = Output::getNumOutputs();
  
  for (int i = 0; i < n; i++) {
    
    Output *o = Output::getOutput(i);
    cbOutput->insertItem(o->getName(), i);
  }


  cbOutput->setCurrentItem(selected);

  return;
}



void InstrumentSelector::update(Subject *s) {

  // Hackish: Just check if the output list needs updating

  if (s == &Output::outputSubject) {

    buildOutputList();
    return;
  }

  if (s != instrument) return;

  int c;

  c = instrument->getMidiChannel();
  if (c == -1) c = 16;
  if (c != cbChan->currentItem()) cbChan->setCurrentItem(c);

  c = instrument->getOutputNum();
  if (c != cbOutput->currentItem()) cbOutput->setCurrentItem(c);

  if (leName->text() != instrument->getName()) 
    leName->setText(instrument->getName());
}
