#ifndef KEYMAPWIDGET_H
#define KEYMAPWIDGET_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qpainter.h>

#include "simsam.h"
#include "observer.h"
#include "sample.h"
#include "midi.h"

class KeymapWidget : public QWidget, public Observer {

  Q_OBJECT;

 private:

  Sample* sample;
  Instrument* instrument;
  SampleMap* sampleMap;

  QPixmap* barFragments[10];
  QPixmap* buffer;
  QPixmap* background;
  QPixmap* marker;
  QPixmap* rootKeyMarker;

  int icons[120];
  int xOffset;

  int selectedKey;

  int playedChannel;
  int playedNote;

 public:

  KeymapWidget(QWidget* parent = NULL, const char* name = NULL);

  void update(Subject* s);
  void subjectDeleted(Subject* s);

  void renderBuffer();
  void buildIconArray();

  // QWidget stuff
  void paintEvent(QPaintEvent *e);
  void mouseMoveEvent(QMouseEvent *e);
  void mousePressEvent(QMouseEvent *e);
  void mouseReleaseEvent(QMouseEvent *e);
  void wheelEvent(QWheelEvent *e);

  void dragEnterEvent(QDragEnterEvent* e);
  void dropEvent(QDropEvent* e);

  public slots:

  void setSample(Sample* s);
  void setInstrument(Instrument* i);

  signals:
  
  void sampleChanged(Sample* s);
 
};

extern KeymapWidget *keymapWidget;

#endif // KEYMAPWIDGET_H
