#ifndef KNOBWIDGET_H
#define KNOBWIDGET_H

#include <qwidget.h>
#include <qdial.h>
#include <qpainter.h>
#include <qpixmap.h>

#include <math.h>

class KnobWidget : public QWidget {

  Q_OBJECT;

  QPixmap *body;
  QPixmap *indicator;

  int value;

  int minValue;
  int maxValue;

  int iconWidth;
  int iconHeight;

  int inX;
  int inY;
  
  int inOffsetX;
  int inOffsetY;

  float radius;

  void calcIndicatorPos();

  float x1;
  float y1;
  
  int stepSize;

 public:

  KnobWidget(QPixmap *b, QPixmap *i, 
	     int w, int h, 
	     int ox, int oy,
	     float r, 
	     QWidget *parent = NULL, const char *name = NULL);

  void paintEvent(QPaintEvent *e);
  
  void wheelEvent(QWheelEvent *e);
  
  void mousePressEvent(QMouseEvent *e);
  void mouseReleaseEvent(QMouseEvent *e);
  void mouseMoveEvent(QMouseEvent *e);
  
  void setRange(int a, int b);

  int getValue();

  public slots:

  void setValue(int v);

  signals:

  void valueChanged(int v);
};


class Knob25 : public KnobWidget {
  
  Q_OBJECT;

 public:

  static QPixmap *body25;
  static QPixmap *indicator25;
  
  Knob25(QWidget *parent = NULL, const char *name = NULL);
};

class Knob35 : public KnobWidget {

  Q_OBJECT;


 public:

  static QPixmap *body35;
  static QPixmap *indicator35;
  
  Knob35(QWidget *parent = NULL, const char *name = NULL);
}; 


#endif // KNOBWIDGET_H
