#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/wait.h>

#include <qapplication.h>
#include <qobject.h>
#include <qaction.h>
#include <qmainwindow.h>
#include <qmenudata.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qlistbox.h>

#include "simsamgui.h"
#include "sampleview.h"
#include "adsrwidget.h"
#include "keymapwidget.h"
#include "knobwidget.h"

#include "simsam.h"
#include "output.h"
#include "midi.h"
#include "sample.h"
#include "settings.h"
#include "error.h"

QApplication *qApplication;

void usage() {

  printf("\n"\
	 "usage: simsam [-v][-h][-a output][-C infile outfile] [file]             \n"\
	 "                                                                        \n"\
	 "  -v                      show version                                  \n"\
	 "                                                                        \n"\
	 "  -h                      display help (this text)                      \n"\
	 "                                                                        \n"\
	 "  -a <output>             select audio output system. Options are       \n"\
	 "                          'alsa' (default) and 'jack'                   \n"\
	 "                                                                        \n"\
	 "  [file]                  patch file to load                            \n"\
	 "\n");
}



int main(int argc, char **argv) {

  int c;

  Settings::load();

  while ((c = getopt(argc, argv, "vha:C:")) != -1) {
    
    switch(c) {    
      
    case 'a':      
      Settings::outputMethod = optarg;
      break;
      
    case 'h':      
      usage();
      return(0);

    case 'v':
      printf("simsam v" VERSION "\n");
      return(0);

    default:
      usage();
      exit(1);
    }
  }
  
  /* init simsam */
  if (Simsam::init() != 0) exit(2);

  if (optind < argc) {

    if (Simsam::load(argv[optind]) < 0) exit(4);
  }

  /* create gui and run... */
  qApplication = new QApplication(argc, argv);

  /* create static knob pixmaps */

  Knob25::body25 = new QPixmap(IMAGEDIR "/knob25-01.png");
  if (Knob25::body25->isNull()) 
    error_message("image not found: " IMAGEDIR "/knob25-01.png");

  Knob25::indicator25 = new QPixmap(IMAGEDIR "/knob25-02.png");
  if (Knob25::indicator25->isNull()) 
    error_message("image not found: " IMAGEDIR "/knob25-02.png");

  Knob35::body35 = new QPixmap(IMAGEDIR "/knob35-01.png");
  if (Knob35::body35->isNull()) 
    error_message("image not found: " IMAGEDIR "/knob35-01.png");

  Knob35::indicator35 = new QPixmap(IMAGEDIR "/knob35-02.png");
  if (Knob35::indicator35->isNull()) 
    error_message("image not found: " IMAGEDIR "/knob35-02.png");

  SimsamMainWindow *mainWindow = new SimsamMainWindow();
  mainWindow->show();
  qApplication->setMainWidget(mainWindow);
  
  return(qApplication->exec());
}








