#include <qlineedit.h>
#include <qlistbox.h>
#include <qerrormessage.h>

#include "outputdialog.h"
#include "nameoutputdialog.uic.h"

#include "output.h"

/* 
 *  Constructs a OutputDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
OutputDialog::OutputDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : OutputDialogUI( parent, name, modal, fl )
{

  buildOutputList();
  
  connect(pbAdd, SIGNAL(clicked()), this, SLOT(addOutput()));
  connect(pbRemove, SIGNAL(clicked()), this, SLOT(removeOutput()));

}

/*  
 *  Destroys the object and frees any allocated resources
 */
OutputDialog::~OutputDialog()
{
    // no need to delete child widgets, Qt does it all for us
}


void OutputDialog::addOutput() {

  NameOutputDialogUI *d = new NameOutputDialogUI();
  QErrorMessage *em = new QErrorMessage(this);

  if (d->exec() == QDialog::Accepted) {

    int res = Output::addOutput(d->leName->text());

    switch(res) {

    case 0:
      buildOutputList();
      break;

    case Output::ERR_MAX_OUTPUTS:
      
      em->message("The current output module cannot create any more outputs!");
      break;

    case Output::ERR_OTHER:
      
      em->message("An error occured while trying to create the output!");
      break;
    }
  }
}

void OutputDialog::removeOutput() {
  
  int i = lbOutputs->currentItem();
  
  if (i == 0) {

    QErrorMessage *em = new QErrorMessage(this);
    em->message("You cannot remove the default output!");

  } else {

    Output::removeOutput(i);
    buildOutputList();
    
  }
}

void OutputDialog::buildOutputList() {
  
  lbOutputs->clear();
  
  int n = Output::getNumOutputs();
  
  for (int i = 0; i < n; i++) {
    
    Output *o = Output::getOutput(i);
    lbOutputs->insertItem(o->getName(), i);
  }
  
  return;
}
