#include "sampleselector.h"

SampleSelector::SampleSelector(QWidget* parent, const char* name, WFlags fl)
  : SampleSelectorUI( parent, name, fl ), Observer(NULL) {

  instrument = Instrument::getInstrument(0);
  instrument->addObserver(this);
  
  buildSampleList();
  
  connect(lbSamples, SIGNAL(highlighted(int)), this, SLOT(selectSample(int)));
  connect(pbAdd, SIGNAL(clicked()), this, SLOT(addSample()));
  connect(pbDel, SIGNAL(clicked()), this, SLOT(delSample()));
}


SampleSelector::~SampleSelector() {
  // no need to delete child widgets, Qt does it all for us
}


void SampleSelector::buildSampleList() {

  int selected = lbSamples->currentItem();

  lbSamples->clear();
  
  int n = instrument->getNumSamples();
  
  for (int i = 0; i < n; i++) {

    Sample* s = instrument->getSample(i);
    lbSamples->insertItem(s->name, i);
  }
  
  if (selected >= n) selected = n - 1;

  lbSamples->setCurrentItem(selected);
}


void SampleSelector::update(Subject* s) {

  if (s == instrument) {

    buildSampleList();
  }
}

void SampleSelector::subjectDeleted(Subject* s) {
}


void SampleSelector::setInstrument(Instrument* i) {

  instrument->removeObserver(this);
  instrument = i;
  instrument->addObserver(this);

  buildSampleList();
}


void SampleSelector::selectSample(int i) {

  emit(sampleSelected(instrument->getSample(i)));
}


void SampleSelector::addSample() {

  instrument->addSample();
}

void SampleSelector::delSample() {

  int i = lbSamples->currentItem();
  Sample* s = instrument->getSample(i);

  instrument->removeSample(s);
}


