#ifndef SIMSAMGUI_H
#define SIMSAMGUI_H
#include "simsamgui.uic.h"

#include <qmainwindow.h>
#include <qmenudata.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qlistbox.h>

#include "sample.h"
#include "observer.h"
#include "simsam.h"
#include "instrument.h"

class SimsamGui : public SimsamGuiUI, public Observer
{ 

private:

  Q_OBJECT;

  QPixmap* backgroundPixmap;

  Sample* sample;
  Instrument* instrument;

  int playedChannel;
  int playedNote;
  
 public:
  SimsamGui(QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
  ~SimsamGui();

  /**
   * Observer methods
   */
  void subjectDeleted(Subject *s);
  void update(Subject *s);

 public slots:
    
  void setSample(Sample* s);
  void setInstrument(Instrument* i);
  
  void loadSample();
  void setName();
  void setGain(int);
  void setPan(int);
  void setSampleStart(int);
  void setSampleEnd(int);
  void setLoopStart(int);
  void setLoopEnd(int);
  void setLoop(bool);
  void setMiddleLoop(bool val);
  void setTrigger(bool);
  void setExclusive(bool);
  void setReverse(bool on);
  void setMidiNote(int v);
  void setModePreset(int t);
  
  void setMasterVolume(int);
  void learnMidi();
 
  void playToggled(bool on);
  void showWaveToggled(bool on);
  
  void panicClicked();
};

class QMainWindow;

class SimsamMainWindow : public QMainWindow {

  Q_OBJECT;

 private:

  SimsamGui *gui;

public:
 
  SimsamMainWindow();
  ~SimsamMainWindow();

private slots:

  void load();
  void save();
  void outputConfig();
  //void guiConfig();
};


#endif // SIMSAMGUI_H
