#include "instrument.h"
#include "output.h"
#include "error.h"
#include "midi.h"

#include <list>
#include <string>

#include <algorithm>

Instrument Instrument::instruments[16];

int Instrument::currentId = 0;

int Instrument::init() {

  char buffer[64];

  for (int i = 0; i < 16; i++) {

    snprintf(buffer, 64, "instrument %.02d", i + 1);  

    instruments[i].setName(strdup(buffer));
    instruments[i].setOutputNum(0);
  }

  instruments[0].setMidiChannel(0);

  return(0);
}

Instrument* Instrument::getInstrument(int iNum) {

  if (iNum >= 0 && iNum < 16) return(&instruments[iNum]);
  return(NULL);
}

Instrument::Instrument() {

  outputNum = 0;
  midiChannel = -1;

  output = NULL;
}



void Instrument::unload() {

  for (int i = 0; i < 16; i++) instruments[i].removeAllSamples();
}


const char* Instrument::getName() { 

  return(name); 
}


void Instrument::setName(const char *n) { 

  strncpy((char*)name, n, 127);

  notify();
}


int Instrument::getOutputNum() { 

  return(outputNum); 
}


void Instrument::setOutputNum(int n) {

  outputNum = n;

  notify();
}


void Instrument::addSample() {

  Sample *s = new Sample(this);

  addSample(s);
}


void Instrument::addSample(Sample *s) {

  sampleList.push_back(s);

  notify();
}


void Instrument::removeSample(Sample *s) {

  sampleList.remove(*find(sampleList.begin(), sampleList.end(), s));

  sampleMap.clearSample(s);
  delete(s);

  notify();
}


void Instrument::removeAllSamples() {
  
  sampleMap.clearAll();

  int i = sampleList.size();
  
  while(i > 0) {

    Sample* s = sampleList.front();
    sampleList.pop_front();

    delete s;

    i--;
  }
}


int Instrument::getNumSamples() {

  return(sampleList.size());
}


void Instrument::setMidiChannel(int c) {

  Midi::instrumentMap.setInstrument(c, this);
  if (Midi::instrumentMap.getInstrument(c) == this) midiChannel = c;

  notify();
}


int Instrument::getMidiChannel() {
  
  return(midiChannel);
}


Sample* Instrument::getSample(int s) {

  if (s >= sampleList.size()) return(NULL);

  std::list<Sample*>::iterator i = sampleList.begin();
  while(s > 0) { i++; s--; }

  return(*i);
}


SampleMap::SampleMap() {
  
  clearAll();
}


void SampleMap::setSample(int note, Sample* s) {
  
  if (samples[note] != NULL) samples[note]->setMidiNote(-1);
  samples[note] = s;

  notify();
}


Sample* SampleMap::getSample(int note) {
  
  return(samples[note]);
}


void SampleMap::clearSample(Sample* s) {
  
  for (int i = 0; i < 120; i++) if (samples[i] == s) samples[i] = NULL;
  notify();
}


void SampleMap::clearAll() {
  
  memset(samples, 0, sizeof(Sample*) * 120);
  notify();
}






