#ifndef _OUTPUT_H
#define _OUTPUT_H

#include <pthread.h>

#define ALSA_PCM_OLD_HW_PARAMS_API
#define ALSA_PCM_OLD_SW_PARAMS_API
#include <alsa/asoundlib.h>

#include <jack/jack.h>

#include "simsam.h"
#include "voice.h"

#include <list>

class OutputMap;

class Output : public Subject {

  static int type;

public:

  enum {

    ERR_NO_ERROR,
    ERR_MAX_OUTPUTS,
    ERR_OTHER
  };

  virtual ~Output();

  const char* name;
  static int numOutputs;

  const char* getName();
  void setName(const char* nName);

  static Subject outputSubject;

  static OutputMap outputMap;

  static std::list<Output*> outputList;

  static int getNumOutputs();
  
  /**
   *  This must be overloaded by sub modules
   */
  static int addOutput(const char* newName);

  static void removeOutput(Output* o);
  static void removeOutput(int outNum);
  static Output* getOutput(int outNum);
  static Output* getDefaultOutput();

  static float masterVolume;
  static int sampleRate;
  static int bufferLen;

  enum {

    OUTPUT_ALSA,
    OUTPUT_JACK
  };

  static int init(int t);
  
  /**
   *  This function must be implemented by any output module.
   *  It is expected to set at least the first element in outputList.
   *  In case of the ALSA driver, which only supports one stereo out
   *  at the moment, this means that a bogus reference != NULL is set, 
   *  thus Output pointers must NEVER be de-referenced outside of
   *  their respective drivers!!! This shouldn't be a problem because
   *  The Output baseclass doesn't expose any non static members.
   */
  static int subInit();

  /**
   * sub modules must overload this
   */
  static int getMaxOutputs();

};


class OutputAlsa : public Output {

private:

  static pthread_t threadHandle;

  static snd_pcm_t* pcmHandle;
  static snd_pcm_hw_params_t* pcmParams;

  static float* buffer_l;
  static float* buffer_r;
  
  static short* outputBuffer;

  static int write(int frames);
  static void* audioThread (void* vp);  

public:

  OutputAlsa(const char* name);
  ~OutputAlsa();

  static int subInit();
  static int addOutput(const char* newName);
  static int getMaxOutputs();
};


class OutputJack : public Output {

private:

  static jack_client_t* client;

  static int processCallback(jack_nframes_t, void*);
  static int buffersizeCallback(jack_nframes_t, void*);
  static int samplerateCallback(jack_nframes_t, void*);
  static void shutdownCallback(void* );
  
public:

  jack_port_t* portLeft;
  jack_port_t* portRight;

  void processOutput(int nFrames);

  OutputJack(const char* newName);
  ~OutputJack();

  static int subInit();
  static int addOutput(const char* newName);
  static int getMaxOutputs();
};


class OutputMap {

  Output* outputs[16];

 public:

  OutputMap();

  void setOutput(int instrument, Output* o);
  void setOutput(int instrument, int outNum);
  Output* getOutput(int instrument);
  void clearOutput(Output* o);
  void clearAll();
};




#endif // _OUTPUT_H










