#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>

#include <qdom.h>
#include <qfile.h>
#include <qstring.h>

#include "patchfile.h"
#include "error.h"
#include "simsam.h"
#include "sample.h"
#include "instrument.h"

/*
int PatchFile::createPatch(const char *path) {
  
  int res = mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR);
  if (res < 0) { 
    perror("mkdir");
    return(-1);
  }
  
  res = chdir(path);
  if (res < 0) {
    perror("chdir");
    return(-1);
  }
 
  res = mkdir("samples", S_IRUSR | S_IWUSR | S_IXUSR);
  if (res < 0) { 
    perror("mkdir");
    return(-1);
  }
}
*/

int PatchFile::load(const char *filename) {

  QDomDocument doc("patch");
  QFile file(filename);

  if (!file.open(IO_ReadOnly)) failure("file: %s", filename);

  if (!doc.setContent(&file)) {
    file.close();
    failure("file: %s", filename);
  }

  file.close();

  QDomElement docElem = doc.documentElement();
  QDomNodeList sectionList = docElem.childNodes();

  for (int section = 0; section < sectionList.length(); section++) {

    QDomNode n = sectionList.item(section);
    
    if (n.nodeName() == "samples") {
      
      QDomNodeList sList = n.childNodes();
      
      for (int i = 0; i < sList.length(); i++) {
	
	QDomNode sNode = sList.item(i);
	
	int id = 
	  sNode.toElement().attributeNode("id").value().toInt();
	int instrumentNum = 
	  sNode.toElement().attributeNode("instrumentNum").value().toInt();
	
	const char *path = NULL;
	const char *label = NULL;
	
	int sampleStart = 0;
	int sampleEnd = 0;
	int loopStart = 0;
	int loopEnd = 0;
	
	int gain = 0;
	int pan = 0;
	
	int loop = 0;
	int middleLoop = 0;
	int trigger = 0;
	
	int reverse = 0;
	int exclusive = 0;
	
	int midiNote = -2;
	int midiRangeStart = -2;
	int midiRangeEnd = -2;
	
	int attackLevel = 100;
	int sustainLevel = 100;
	int attackTime = 0;
	int decayTime = 0;
	int releaseTime = 5;
	
	QDomNodeList pList = sNode.childNodes();
	
	for (int j = 0; j < pList.length(); j++) {
	  
	  QDomNode pNode = pList.item(j);
	  QDomElement pElem = pNode.toElement();
	  
	  QString tagName = pElem.tagName();
	  
	  if (tagName == "path") {
	    
	    path = pElem.text();
	    
	  } else if (tagName == "label") {
	    
	    label = pElem.text();
	    
	  } else if (tagName == "sampleStart") {
	    
	    sampleStart = pElem.text().toInt();
	    
	  } else if (tagName == "sampleEnd") {
	    
	    sampleEnd = pElem.text().toInt();
	    
	  } else if (tagName == "loopStart") {
	    
	    loopStart = pElem.text().toInt();
	    
	  } else if (tagName == "loopEnd") {
	    
	    loopEnd = pElem.text().toInt();
	    
	  } else if (tagName == "gain") {
	    
	    gain = pElem.text().toInt();
	    
	  } else if (tagName == "pan") {
	    
	    pan = pElem.text().toInt();
	    
	  } else if (tagName == "loop") {
	    
	    loop = pElem.text().toInt();
	    
	  } else if (tagName == "middleLoop") {
	    
	    middleLoop = pElem.text().toInt();
	    
	  } else if (tagName == "trigger") {
	    
	    trigger = pElem.text().toInt();
	    
	  } else if (tagName == "exclusive") {
	    
	    exclusive = pElem.text().toInt();
	    
	  } else if (tagName == "reverse") {
	    
	    reverse = pElem.text().toInt();
	    
	  } else if (tagName == "midiNote") {
	    
	    midiNote = pElem.text().toInt();
	    
	  } else if (tagName == "midiRangeStart") {
	    
	    midiRangeStart = pElem.text().toInt();
	    
	  } else if (tagName == "midiRangeEnd") {
	    
	    midiRangeEnd = pElem.text().toInt();
	    
	  } else if (tagName == "attackTime") {
	    
	    attackTime = pElem.text().toInt();
	    
	  } else if (tagName == "decayTime") {
	    
	    decayTime =  pElem.text().toInt();
	    
	  } else if (tagName == "releaseTime") {
	    
	    releaseTime = pElem.text().toInt();
	    
	  } else if (tagName == "attackLevel") {
	    
	    attackLevel = pElem.text().toInt();
	    
	  } else if (tagName == "sustainLevel") {
	    
	    sustainLevel = pElem.text().toInt();
	  }
	}
	
	Instrument* instrument = Instrument::getInstrument(instrumentNum);
	if (!instrument) instrument = Instrument::getInstrument(0);
	
	Sample *s = new Sample(instrument);
	
	int res = s->loadFile(path);
	if (res < 0) { 
	  
	  delete s;
	  break;
	}
	
	strncpy(s->name, label, 256);
	
	s->setSampleStart(sampleStart);
	s->setSampleEnd(sampleEnd);
	s->setLoopStart(loopStart);
	s->setLoopEnd(loopEnd);
	
	s->setGain(gain);
	s->setPan(pan);
	
	s->setLoop(loop != 0);
	s->setMiddleLoop(middleLoop != 0);
	s->setTrigger(trigger != 0);
	s->setExclusive(exclusive != 0);
	s->setReverse(reverse != 0);
	
	s->setMidiNote(midiNote);
	s->setMidiRangeStart(midiRangeStart);
	s->setMidiRangeEnd(midiRangeEnd);
	
	s->adsr.setAttackLevel(attackLevel);
	s->adsr.setSustainLevel(sustainLevel);
	
	s->adsr.setAttackTime(attackTime);
	s->adsr.setDecayTime(decayTime);
	s->adsr.setReleaseTime(releaseTime);
	
	instrument->addSample(s);
	
      }
      
    } else if (n.nodeName() == "instruments") {
      
      QDomNodeList sList = n.childNodes();
      
      int n = sList.length();
      if (n > 16) n = 16;
      
      for (int i = 0; i < n; i++) {
	
	QDomNode sNode = sList.item(i);
	
	const char* name = "no name";
	int outputNum = 0;
	int midiChannel = -1;
	
	QDomNodeList pList = sNode.childNodes();
	
	for (int j = 0; j < pList.length(); j++) {
	  
	  QDomNode pNode = pList.item(j);
	  QDomElement pElem = pNode.toElement();
	  
	  QString tagName = pElem.tagName();
	  
	  if (tagName == "name") {
	    
	    name = pElem.text();
	    
	  } else if (tagName == "midiChannel") {
	    
	    midiChannel = pElem.text().toInt();
	    
	  } else if (tagName == "outputNum") {
	    
	    outputNum = pElem.text().toInt();
	  } 
	}
	
	Instrument* instrument = Instrument::getInstrument(i);
	
	instrument->setName(name);
	instrument->setMidiChannel(midiChannel);
	instrument->setOutputNum(outputNum);
      }
    }
  }    

  return(0);
}


int PatchFile::save(const char *filename) {
  
  QDomDocument doc("patch");
  QDomElement root = doc.createElement("patch");
  doc.appendChild(root);

  root.setAttribute("version", 1);
  
  // save samples

  QDomElement samplesElem = doc.createElement("samples");
  root.appendChild(samplesElem);

  for (int i = 0; i < 16; i++) {

    int numSamples = Instrument::instruments[i].getNumSamples();

    for (int j = 0; j < numSamples; j++) {

      Sample *s = Instrument::instruments[i].getSample(j);
      if (!s || !s->isValid()) break;

      QDomElement sElem = doc.createElement("sample");
      samplesElem.appendChild(sElem);
      
      sElem.setAttribute("id", s->id);
      sElem.setAttribute("instrumentNum", i);
	
      QString qs;
      QDomElement pElem;
      QDomText pText;
	
      pElem = doc.createElement("path");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(s->path);
      pElem.appendChild(pText);
	
      pElem = doc.createElement("label");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(s->name);
      pElem.appendChild(pText);
	
      pElem = doc.createElement("sampleStart");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getSampleStart()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("sampleEnd");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getSampleEnd()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("loopStart");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getLoopStart()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("loopEnd");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getLoopEnd()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("loop");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getLoop()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("middleLoop");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getMiddleLoop()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("trigger");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getTrigger()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("exclusive");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getExclusive()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("reverse");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getReverse()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("gain");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getGain()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("pan");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getPan()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("attackTime");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->adsr.getAttackTime()));
      pElem.appendChild(pText);
      pElem = doc.createElement("decayTime");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->adsr.getDecayTime()));
      pElem.appendChild(pText);
      pElem = doc.createElement("releaseTime");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->adsr.getReleaseTime()));
      pElem.appendChild(pText);
      pElem = doc.createElement("attackLevel");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->adsr.getAttackLevel()));
      pElem.appendChild(pText);
      pElem = doc.createElement("sustainLevel");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->adsr.getSustainLevel()));
      pElem.appendChild(pText);
      
      pElem = doc.createElement("midiNote");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getMidiNote()));
      pElem.appendChild(pText);
	
      pElem = doc.createElement("midiRangeStart");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getMidiRangeStart()));
      pElem.appendChild(pText);
      pElem = doc.createElement("midiRangeEnd");
      sElem.appendChild(pElem);
      pText = doc.createTextNode(qs.sprintf("%d", s->getMidiRangeEnd()));
      pElem.appendChild(pText);
    }
  }
  
  // save instruments
  
  QDomElement instrumentsElem = doc.createElement("instruments");
  root.appendChild(instrumentsElem);
  
  for (int i = 0; i < 16; i++) {

    Instrument *instrument = Instrument::getInstrument(i);
    
    QDomElement iElem = doc.createElement("instrument");
    instrumentsElem.appendChild(iElem);
    
    QString qs;
    QDomElement pElem;
    QDomText pText;
    
    pElem = doc.createElement("name");
    iElem.appendChild(pElem);
    pText = doc.createTextNode(instrument->getName());
    pElem.appendChild(pText);
    
    pElem = doc.createElement("midiChannel");
    iElem.appendChild(pElem);
    pText = doc.createTextNode(qs.sprintf("%d", instrument->getMidiChannel()));
    pElem.appendChild(pText);

    pElem = doc.createElement("outputNum");
    iElem.appendChild(pElem);
    pText = doc.createTextNode(qs.sprintf("%d", instrument->getOutputNum()));
    pElem.appendChild(pText);
  }
  
  FILE *fp = fopen(filename, "w+");
  if(!fp) {
    perror("fopen");
    return(-1);
  }
  
  fprintf(fp, "%s", (char*) doc.toString().ascii());
  
  fclose(fp);
  
  return(0);
}

/*

int PatchFile::open(const char *path) {

  int res = chdir(path);

  if (res < 0) { 
    perror("chdir");
    return(-1);
  }

  return(0);
}



*/

/* 
$Id: patchfile.cpp,v 1.7 2004-03-30 01:09:12 chenz Exp $
*/










