#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <qdom.h>
#include <qfile.h>

#include "settings.h"
#include "error.h"

const char* Settings::outputMethod = "alsa";

const char* Settings::jackClientName = "simsam";
const char* Settings::jackLeftConnection = "alsa_pcm:playback_1";
const char* Settings::jackRightConnection = "alsa_pcm:playback_2";

const char* Settings::alsaPcmDevice = "default";
int Settings::alsaBufferLength = 1024;
int Settings::alsaSampleRate = 44100;

const char* Settings::alsaSeqClientName = "simsam";
const char* Settings::alsaSeqPortName = "simsam-midi-in";

const char* Settings::samplePath = getenv("HOME");
const char* Settings::patchPath = getenv("HOME");


int Settings::load() {

  char buffer[1024];
  
  memset(buffer, 0, 1024);

  char* homeDir = getenv("HOME");
  
  snprintf(buffer, 1023, "%s/.simsam/settings.xml", homeDir);

  QDomDocument doc("settings");
  QFile file(buffer);

  if (!file.open(IO_ReadOnly)) return(-1);

  if (!doc.setContent(&file)) {
    file.close();
    failure("doc.setContent");
  }

  file.close();

  QDomElement docElem = doc.documentElement();
  QDomNodeList sList = docElem.childNodes();

  for (int i = 0; i < sList.length(); i++) {

    QDomNode node = sList.item(i);
    QDomElement element = node.toElement();

    QString text = element.text();
    QString tagName = element.tagName();

    if (tagName == "outputMethod") {
      
      outputMethod = strdup(text);

    } else if (tagName == "jackClientName") {

      jackClientName = strdup(text);

    } else if (tagName == "alsaPcmDevice") {
    
      alsaPcmDevice = strdup(text);

    } else if (tagName =="jackLeftConnection") {

      jackLeftConnection = strdup(text);

    } else if (tagName =="jackRightConnection") {

      jackRightConnection = strdup(text);

    } else if (tagName =="alsaSeqClientName") {

      alsaSeqClientName = strdup(text);

   } else if (tagName =="alsaSeqPortName") {

      alsaSeqPortName = strdup(text);

    } else if (tagName =="samplePath") {

      samplePath = strdup(text);

    } else if (tagName =="patchPath") {

      patchPath = strdup(text);

    } else if (tagName =="alsaBufferLength") {

      alsaBufferLength = text.toInt();

    } else if (tagName =="alsaSampleRate") {

      alsaSampleRate = text.toInt();
     
    }
  }

  return(0);
}






