#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <pthread.h>

#include "simsam.h"
#include "midi.h"
#include "output.h"
#include "sample.h"
#include "settings.h"
#include "error.h"
#include "patchfile.h"
#include "instrument.h"

bool Simsam::framesToMs(int frames, int *ms) {

  if (Output::sampleRate <= 0) return(false);
  float factor = 1000.0f / (float)Output::sampleRate;

  *ms = (int)(frames * factor);

  return(true);
}

bool Simsam::msToFrames(int ms, int *frames) {

  if (Output::sampleRate <= 0) return(false);
  float factor = 1000.0f / (float)Output::sampleRate;

  *frames = (int)(ms / factor);
  return(true);
}


int Simsam::init() {

  int r = -1;

  if (strcmp(Settings::outputMethod, "jack") == 0) { 

    r = Output::init(Output::OUTPUT_JACK);

  } else if (strcmp(Settings::outputMethod, "alsa") == 0) {

    r = Output::init(Output::OUTPUT_ALSA);

  } else {
    
    error_message("unknown output method '%s'!",Settings::outputMethod);
    return(-1);
  }
  if (r < 0) { error_message("output initialization failed!"); return(-1); }

  r = Midi::init();
  if (r < 0) { error_message("midi initialization failed!"); return(-1); }

  r  = Instrument::init();
  if (r < 0) { error_message("instrument initialization failed!"); return(-1); }
  return(0);
}


int Simsam::load(const char *filename) {

  // stop all voices
  panic();

  // unload samples
  Instrument::unload();

  return(PatchFile::load(filename));
}


int Simsam::save(const char *filename) {

  return(PatchFile::save(filename));
}


pthread_mutex_t Simsam::mutex = PTHREAD_MUTEX_INITIALIZER;

void Simsam::lock() {

  pthread_mutex_lock(&mutex);
}

void Simsam::unlock() {

  pthread_mutex_unlock(&mutex);
}

void Simsam::panic() {

  Voice::panic();
}

