#ifndef VOICE_H
#define VOICE_H

#include "sample.h"
#include "envelope.h"
#include "output.h"

class Output;

class Voice {

  Sample *sample;
  int readIndex;

  EnvADSR adsr;

  float velocity;

  int state;

  int note;
  int noteOffset;

  int channel;

  Output *output;

 public:

  // the global array of voices
  static Voice voices[SIMSAM_MAX_VOICES];

  enum {
    
    OFF,
    ALLOCATED,
    ON,
    RELEASED
  };
  
  static int voiceCnt;

  static void incVoiceCnt();
  static void decVoiceCnt();

  static Voice *allocateVoice(Sample *s, int outNum);
  static Voice *getVoice(int channel, int note);

  static void makeExclusive(int channel, Voice *v);
  static void panic();

  Voice();
  bool allocate(Sample *s, int outNum);
  void release();

  void addToMix(float *l, float *r, int len);

  bool isExclusive();
  bool isUsed();

  void setState(int s);

  void noteOn(int channel, int note, int vel);
  void noteOff();

  int getNote();
  int getChannel();
  Output* getOutput();
  
};


#endif // VOICE_H

/*
  $Id: voice.h,v 1.4 2004-02-17 16:53:36 chenz Exp $
*/
